# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: brokrpc/spec/v1/consumer.proto
# Protobuf Python Version: 5.27.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    1,
    '',
    'brokrpc/spec/v1/consumer.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1e\x62rokrpc/spec/v1/consumer.proto\x12\x0f\x62rokrpc.spec.v1\x1a google/protobuf/descriptor.proto\"9\n\x0f\x43onsumerOptions\x12\x1b\n\x06\x66\x61\x64ing\x18\x01 \x01(\x08H\x00R\x06\x66\x61\x64ing\x88\x01\x01\x42\t\n\x07_fading:`\n\x08\x63onsumer\x12\x1e.google.protobuf.MethodOptions\x18\xa9\x46 \x01(\x0b\x32 .brokrpc.spec.v1.ConsumerOptionsR\x08\x63onsumer\x88\x01\x01\x42\x82\x01\n\x13\x63om.brokrpc.spec.v1B\rConsumerProtoP\x01\xa2\x02\x03\x42SX\xaa\x02\x0f\x42rokrpc.Spec.V1\xca\x02\x0f\x42rokrpc\\Spec\\V1\xe2\x02\x1b\x42rokrpc\\Spec\\V1\\GPBMetadata\xea\x02\x11\x42rokrpc::Spec::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'brokrpc.spec.v1.consumer_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\023com.brokrpc.spec.v1B\rConsumerProtoP\001\242\002\003BSX\252\002\017Brokrpc.Spec.V1\312\002\017Brokrpc\\Spec\\V1\342\002\033Brokrpc\\Spec\\V1\\GPBMetadata\352\002\021Brokrpc::Spec::V1'
  _globals['_CONSUMEROPTIONS']._serialized_start=85
  _globals['_CONSUMEROPTIONS']._serialized_end=142
# @@protoc_insertion_point(module_scope)
