# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: brokrpc/spec/v1/amqp.proto
# Protobuf Python Version: 5.27.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    1,
    '',
    'brokrpc/spec/v1/amqp.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x62rokrpc/spec/v1/amqp.proto\x12\x0f\x62rokrpc.spec.v1\x1a google/protobuf/descriptor.proto\"I\n\x11\x41rgumentListValue\x12\x34\n\x05items\x18\x01 \x03(\x0b\x32\x1e.brokrpc.spec.v1.ArgumentValueR\x05items\"\xb0\x01\n\x10\x41rgumentMapValue\x12\x42\n\x05items\x18\x01 \x03(\x0b\x32,.brokrpc.spec.v1.ArgumentMapValue.ItemsEntryR\x05items\x1aX\n\nItemsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32\x1e.brokrpc.spec.v1.ArgumentValueR\x05value:\x02\x38\x01\"\xc2\x02\n\rArgumentValue\x12\x1f\n\nnull_value\x18\x01 \x01(\x08H\x00R\tnullValue\x12\x1f\n\nbool_value\x18\x02 \x01(\x08H\x00R\tboolValue\x12\x1d\n\tint_value\x18\x03 \x01(\x03H\x00R\x08intValue\x12!\n\x0b\x66loat_value\x18\x04 \x01(\x02H\x00R\nfloatValue\x12\x1d\n\tstr_value\x18\x05 \x01(\tH\x00R\x08strValue\x12\x43\n\nlist_value\x18\x06 \x01(\x0b\x32\".brokrpc.spec.v1.ArgumentListValueH\x00R\tlistValue\x12@\n\tmap_value\x18\x07 \x01(\x0b\x32!.brokrpc.spec.v1.ArgumentMapValueH\x00R\x08mapValueB\x07\n\x05value\"\x82\x03\n\x0f\x45xchangeOptions\x12\x17\n\x04name\x18\x01 \x01(\tH\x00R\x04name\x88\x01\x01\x12\x36\n\x04type\x18\x02 \x01(\x0e\x32\x1d.brokrpc.spec.v1.ExchangeTypeH\x01R\x04type\x88\x01\x01\x12\x1d\n\x07\x64urable\x18\x03 \x01(\x08H\x02R\x07\x64urable\x88\x01\x01\x12$\n\x0b\x61uto_delete\x18\x04 \x01(\x08H\x03R\nautoDelete\x88\x01\x01\x12M\n\targuments\x18\x05 \x03(\x0b\x32/.brokrpc.spec.v1.ExchangeOptions.ArgumentsEntryR\targuments\x1a\\\n\x0e\x41rgumentsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32\x1e.brokrpc.spec.v1.ArgumentValueR\x05value:\x02\x38\x01\x42\x07\n\x05_nameB\x07\n\x05_typeB\n\n\x08_durableB\x0e\n\x0c_auto_delete\"\xec\x02\n\x0cQueueOptions\x12\x17\n\x04name\x18\x01 \x01(\tH\x00R\x04name\x88\x01\x01\x12\x1d\n\x07\x64urable\x18\x02 \x01(\x08H\x01R\x07\x64urable\x88\x01\x01\x12!\n\texclusive\x18\x03 \x01(\x08H\x02R\texclusive\x88\x01\x01\x12$\n\x0b\x61uto_delete\x18\x04 \x01(\x08H\x03R\nautoDelete\x88\x01\x01\x12J\n\targuments\x18\x05 \x03(\x0b\x32,.brokrpc.spec.v1.QueueOptions.ArgumentsEntryR\targuments\x1a\\\n\x0e\x41rgumentsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32\x1e.brokrpc.spec.v1.ArgumentValueR\x05value:\x02\x38\x01\x42\x07\n\x05_nameB\n\n\x08_durableB\x0c\n\n_exclusiveB\x0e\n\x0c_auto_delete*\x94\x01\n\x0c\x45xchangeType\x12\x1d\n\x19\x45XCHANGE_TYPE_UNSPECIFIED\x10\x00\x12\x18\n\x14\x45XCHANGE_TYPE_DIRECT\x10\x01\x12\x18\n\x14\x45XCHANGE_TYPE_FANOUT\x10\x02\x12\x17\n\x13\x45XCHANGE_TYPE_TOPIC\x10\x03\x12\x18\n\x14\x45XCHANGE_TYPE_HEADER\x10\x04:a\n\x08\x65xchange\x12\x1f.google.protobuf.ServiceOptions\x18\xa8, \x01(\x0b\x32 .brokrpc.spec.v1.ExchangeOptionsR\x08\x65xchange\x88\x01\x01:W\n\x05queue\x12\x1e.google.protobuf.MethodOptions\x18\xa8, \x01(\x0b\x32\x1d.brokrpc.spec.v1.QueueOptionsR\x05queue\x88\x01\x01\x42~\n\x13\x63om.brokrpc.spec.v1B\tAmqpProtoP\x01\xa2\x02\x03\x42SX\xaa\x02\x0f\x42rokrpc.Spec.V1\xca\x02\x0f\x42rokrpc\\Spec\\V1\xe2\x02\x1b\x42rokrpc\\Spec\\V1\\GPBMetadata\xea\x02\x11\x42rokrpc::Spec::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'brokrpc.spec.v1.amqp_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\023com.brokrpc.spec.v1B\tAmqpProtoP\001\242\002\003BSX\252\002\017Brokrpc.Spec.V1\312\002\017Brokrpc\\Spec\\V1\342\002\033Brokrpc\\Spec\\V1\\GPBMetadata\352\002\021Brokrpc::Spec::V1'
  _globals['_ARGUMENTMAPVALUE_ITEMSENTRY']._loaded_options = None
  _globals['_ARGUMENTMAPVALUE_ITEMSENTRY']._serialized_options = b'8\001'
  _globals['_EXCHANGEOPTIONS_ARGUMENTSENTRY']._loaded_options = None
  _globals['_EXCHANGEOPTIONS_ARGUMENTSENTRY']._serialized_options = b'8\001'
  _globals['_QUEUEOPTIONS_ARGUMENTSENTRY']._loaded_options = None
  _globals['_QUEUEOPTIONS_ARGUMENTSENTRY']._serialized_options = b'8\001'
  _globals['_EXCHANGETYPE']._serialized_start=1417
  _globals['_EXCHANGETYPE']._serialized_end=1565
  _globals['_ARGUMENTLISTVALUE']._serialized_start=81
  _globals['_ARGUMENTLISTVALUE']._serialized_end=154
  _globals['_ARGUMENTMAPVALUE']._serialized_start=157
  _globals['_ARGUMENTMAPVALUE']._serialized_end=333
  _globals['_ARGUMENTMAPVALUE_ITEMSENTRY']._serialized_start=245
  _globals['_ARGUMENTMAPVALUE_ITEMSENTRY']._serialized_end=333
  _globals['_ARGUMENTVALUE']._serialized_start=336
  _globals['_ARGUMENTVALUE']._serialized_end=658
  _globals['_EXCHANGEOPTIONS']._serialized_start=661
  _globals['_EXCHANGEOPTIONS']._serialized_end=1047
  _globals['_EXCHANGEOPTIONS_ARGUMENTSENTRY']._serialized_start=909
  _globals['_EXCHANGEOPTIONS_ARGUMENTSENTRY']._serialized_end=1001
  _globals['_QUEUEOPTIONS']._serialized_start=1050
  _globals['_QUEUEOPTIONS']._serialized_end=1414
  _globals['_QUEUEOPTIONS_ARGUMENTSENTRY']._serialized_start=909
  _globals['_QUEUEOPTIONS_ARGUMENTSENTRY']._serialized_end=1001
# @@protoc_insertion_point(module_scope)
