# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: brokrpc/spec/v1/call.proto
# Protobuf Python Version: 5.27.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    1,
    '',
    'brokrpc/spec/v1/call.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.rpc import code_pb2 as google_dot_rpc_dot_code__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x62rokrpc/spec/v1/call.proto\x12\x0f\x62rokrpc.spec.v1\x1a\x19google/protobuf/any.proto\x1a\x15google/rpc/code.proto\"b\n\x05\x45rror\x12\x1d\n\x07message\x18\x01 \x01(\tH\x00R\x07message\x88\x01\x01\x12.\n\x07\x64\x65tails\x18\x02 \x03(\x0b\x32\x14.google.protobuf.AnyR\x07\x64\x65tailsB\n\n\x08_message\"R\n\x0cUnaryRequest\x12\x12\n\x04uuid\x18\x01 \x01(\x0cR\x04uuid\x12.\n\x07payload\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyR\x07payload\"\xc7\x01\n\rUnaryResponse\x12\x12\n\x04uuid\x18\x01 \x01(\x0cR\x04uuid\x12$\n\x04\x63ode\x18\x02 \x01(\x0e\x32\x10.google.rpc.CodeR\x04\x63ode\x12\x33\n\x07payload\x18\x03 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00R\x07payload\x88\x01\x01\x12\x31\n\x05\x65rror\x18\x04 \x01(\x0b\x32\x16.brokrpc.spec.v1.ErrorH\x01R\x05\x65rror\x88\x01\x01\x42\n\n\x08_payloadB\x08\n\x06_error\"\xbd\x02\n\rStreamRequest\x12\x12\n\x04uuid\x18\x01 \x01(\x0cR\x04uuid\x12\x14\n\x05index\x18\x02 \x01(\x04R\x05index\x12<\n\x05start\x18\x03 \x01(\x0b\x32$.brokrpc.spec.v1.StreamRequest.StartH\x00R\x05start\x12\x39\n\x04part\x18\x04 \x01(\x0b\x32#.brokrpc.spec.v1.StreamRequest.PartH\x00R\x04part\x12\x36\n\x03\x65nd\x18\x05 \x01(\x0b\x32\".brokrpc.spec.v1.StreamRequest.EndH\x00R\x03\x65nd\x1a\x07\n\x05Start\x1a\x38\n\x04Part\x12\x30\n\x08payloads\x18\x01 \x03(\x0b\x32\x14.google.protobuf.AnyR\x08payloads\x1a\x05\n\x03\x45ndB\x07\n\x05\x65vent\"\xa4\x03\n\x0eStreamResponse\x12\x12\n\x04uuid\x18\x01 \x01(\x0cR\x04uuid\x12\x14\n\x05index\x18\x02 \x01(\x04R\x05index\x12=\n\x05start\x18\x03 \x01(\x0b\x32%.brokrpc.spec.v1.StreamResponse.StartH\x00R\x05start\x12:\n\x04part\x18\x04 \x01(\x0b\x32$.brokrpc.spec.v1.StreamResponse.PartH\x00R\x04part\x12\x37\n\x03\x65nd\x18\x05 \x01(\x0b\x32#.brokrpc.spec.v1.StreamResponse.EndH\x00R\x03\x65nd\x1a\x07\n\x05Start\x1a\x38\n\x04Part\x12\x30\n\x08payloads\x18\x01 \x03(\x0b\x32\x14.google.protobuf.AnyR\x08payloads\x1ah\n\x03\x45nd\x12$\n\x04\x63ode\x18\x01 \x01(\x0e\x32\x10.google.rpc.CodeR\x04\x63ode\x12\x31\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x16.brokrpc.spec.v1.ErrorH\x00R\x05\x65rror\x88\x01\x01\x42\x08\n\x06_errorB\x07\n\x05\x65ventB~\n\x13\x63om.brokrpc.spec.v1B\tCallProtoP\x01\xa2\x02\x03\x42SX\xaa\x02\x0f\x42rokrpc.Spec.V1\xca\x02\x0f\x42rokrpc\\Spec\\V1\xe2\x02\x1b\x42rokrpc\\Spec\\V1\\GPBMetadata\xea\x02\x11\x42rokrpc::Spec::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'brokrpc.spec.v1.call_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\023com.brokrpc.spec.v1B\tCallProtoP\001\242\002\003BSX\252\002\017Brokrpc.Spec.V1\312\002\017Brokrpc\\Spec\\V1\342\002\033Brokrpc\\Spec\\V1\\GPBMetadata\352\002\021Brokrpc::Spec::V1'
  _globals['_ERROR']._serialized_start=97
  _globals['_ERROR']._serialized_end=195
  _globals['_UNARYREQUEST']._serialized_start=197
  _globals['_UNARYREQUEST']._serialized_end=279
  _globals['_UNARYRESPONSE']._serialized_start=282
  _globals['_UNARYRESPONSE']._serialized_end=481
  _globals['_STREAMREQUEST']._serialized_start=484
  _globals['_STREAMREQUEST']._serialized_end=801
  _globals['_STREAMREQUEST_START']._serialized_start=720
  _globals['_STREAMREQUEST_START']._serialized_end=727
  _globals['_STREAMREQUEST_PART']._serialized_start=729
  _globals['_STREAMREQUEST_PART']._serialized_end=785
  _globals['_STREAMREQUEST_END']._serialized_start=787
  _globals['_STREAMREQUEST_END']._serialized_end=792
  _globals['_STREAMRESPONSE']._serialized_start=804
  _globals['_STREAMRESPONSE']._serialized_end=1224
  _globals['_STREAMRESPONSE_START']._serialized_start=720
  _globals['_STREAMRESPONSE_START']._serialized_end=727
  _globals['_STREAMRESPONSE_PART']._serialized_start=729
  _globals['_STREAMRESPONSE_PART']._serialized_end=785
  _globals['_STREAMRESPONSE_END']._serialized_start=1111
  _globals['_STREAMRESPONSE_END']._serialized_end=1215
# @@protoc_insertion_point(module_scope)
